@@ -21,6 +21,7 @@ package org.apache.royale.compiler.internal.parsing.as;
 
 import java.lang.reflect.Field;
 
+import org.apache.royale.compiler.config.CompilerDiagnosticsConstants;
 import org.apache.royale.compiler.internal.parsing.TokenBase;
 import org.apache.royale.compiler.parsing.IASToken;
 import org.apache.royale.compiler.parsing.ICMToken;
@@ -892,7 +893,11 @@ public class ASToken extends TokenBase implements IASToken, ASTokenTypes
     @Override
     protected String getTypeString()
     {
+    	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASTOKEN) == CompilerDiagnosticsConstants.ASTOKEN)
+    		System.out.println("ASToken waiting for lock for " + getSourcePath() + ":" + getLine() + ":" + getColumn());
         initializeTokenNames();
+    	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASTOKEN) == CompilerDiagnosticsConstants.ASTOKEN)
+    		System.out.println("ASToken done with lock for " + getSourcePath() + ":" + getLine() + ":" + getColumn());
         return tokenNames.get(getType());
     }
 
